package it.unibs.fp.patenteelibbrettopregoh;

import it.kibo.fp.lib.InputData;
import java.util.ArrayList;


/**
 * PatenteELibbrettoPregohMain.java
 * Questa classe rappresenta il punto di ingresso del gioco "Patente e Libretto Pregoh".
 * Inizializza il gioco, gestisce il ciclo di gioco e termina il gioco.
 */
public class PatenteELibbrettoPregohMain {
    /**
     * Metodo principale che avvia il gioco.
     *
     */
    public static void main(String[] args) {
        System.out.println("PatenteELibbrettoPregoh!");

        Formato formato = getFormato();
        GameData gameData = new GameData(formato);
        gameLoop(gameData);
        endGame(gameData);

    }

    /**
     * Metodo che gestisce il ciclo di gioco.
     *
     * @param gameData i dati di gioco
     */
    private static void gameLoop(GameData gameData) {
        while (gameData.getStatoGioco() == StatoGioco.IN_CORSO) {
            int personeChePassano = ((int)(Math.random() * 5) + 3);
            System.out.println("\nIl giorno " + gameData.getData() + " passeranno " + personeChePassano + " persone\nIl tuo saldo è di: " + gameData.getSaldo() + "§\n");
            ArrayList<PersonaControllata> personeControllate = new ArrayList<>();
            for (int i = 0; i < personeChePassano; i++) {

                if (gameData.getStatoGioco() == StatoGioco.IN_CORSO) {
                    System.out.println("\nPassa la persona " + (i + 1) + " di " + personeChePassano + "\n");
                    gestisciPersona(gameData, personeControllate);
                }
            }
            gameData.incrementaGiorno(personeControllate);
        }
    }

    /**
     * Metodo che gestisce una persona controllata.
     *
     * @param gameData i dati di gioco
     * @param personeControllate la lista delle persone controllate
     */
    private static void gestisciPersona(GameData gameData, ArrayList<PersonaControllata> personeControllate) {
        Persona personaDaControllare = gameData.getPersonaDaControlare();
        boolean esito = controllaPersona(personaDaControllare, gameData.getComuni(), gameData);
        boolean accettata = InputData.readYesOrNo("Si presenta\n" + personaDaControllare + "\nC.F. calcolato: " + personaDaControllare.calcolaCodiceFiscale(gameData.getComuni()) + "\nAccettare");
        if (accettata) {
            if (esito){
                datiCorrettiAccettati(personaDaControllare, personeControllate);
            } else {
                datiSbagliatiAccettati(personaDaControllare, personeControllate, gameData);
            }
        } else {
            if (esito) {
                datiCorrettiRifiutati(personaDaControllare, personeControllate, gameData);
            } else {
                datiSbagliatiRifiutati(personaDaControllare, gameData, personeControllate);
            }
        }
    }

    /**
     * Metodo che gestisce i dati errati rifiutati.
     *
     * @param personaDaControllare la persona da controllare
     * @param gameData i dati di gioco
     * @param personeControllate la lista delle persone controllate
     */
    private static void datiSbagliatiRifiutati(Persona personaDaControllare, GameData gameData, ArrayList<PersonaControllata> personeControllate) {
        System.out.println("La persona aveva i dati errati");
        PersonaControllata personaControllata;

        if (Math.random() < 0.4){
            gestisciMazzetta(personaDaControllare, gameData, personeControllate);
        } else {
            personaRifiutata(personaDaControllare, personeControllate);
        }
    }

    /**
     * Metodo che gestisce la persona rifiutata.
     *
     * @param personaDaControllare la persona da controllare
     * @param personeControllate la lista delle persone controllate
     */
    private static void personaRifiutata(Persona personaDaControllare, ArrayList<PersonaControllata> personeControllate) {
        PersonaControllata personaControllata;
        personaControllata = new PersonaControllata(personaDaControllare, false, TipoPersonaControllata.PERSONA_NORMALE);
        personeControllate.add(personaControllata);
    }

    /**
     * Metodo che gestisce la mazzetta.
     *
     * @param personaDaControllare la persona da controllare
     * @param gameData i dati di gioco
     * @param personeControllate la lista delle persone controllate
     */
    private static void gestisciMazzetta(Persona personaDaControllare, GameData gameData, ArrayList<PersonaControllata> personeControllate) {
        PersonaControllata personaControllata;
        int mazzetta = ((int)((Math.random() * 251))) + 250;
        System.out.println(personaDaControllare.getNome() + " " + personaDaControllare.getCognome() + " prova a corromperti con " + mazzetta + "§");
        boolean accettataMazzetta = InputData.readYesOrNo("Accettare la mazzetta");
        if (accettataMazzetta){
            gameData.addSaldo(mazzetta);
            if (Math.random() < 0.7){
                mazzettaRivoluzionario(personaDaControllare, gameData, personeControllate);
            } else {
                mazzettaPoliziotto(personaDaControllare, gameData, personeControllate);
            }
        } else {
            mazzettaRifiutata(personaDaControllare, personeControllate);
        }
    }

    /**
     * Metodo che gestisce la mazzetta rifiutata.
     *
     * @param personaDaControllare la persona da controllare
     * @param personeControllate la lista delle persone controllate
     */
    private static void mazzettaRifiutata(Persona personaDaControllare, ArrayList<PersonaControllata> personeControllate) {
        PersonaControllata personaControllata;
        personaControllata = new PersonaControllata(personaDaControllare, false, TipoPersonaControllata.PERSONA_NORMALE);
        personeControllate.add(personaControllata);
    }

    /**
     * Metodo che gestisce la mazzetta di un poliziotto.
     *
     * @param personaDaControllare la persona da controllare
     * @param gameData i dati di gioco
     * @param personeControllate la lista delle persone controllate
     */
    private static void mazzettaPoliziotto(Persona personaDaControllare, GameData gameData, ArrayList<PersonaControllata> personeControllate) {
        PersonaControllata personaControllata;
        System.out.println("Hai accettato la mazzetta ... era un poliziotto");
        gameData.incrementaMazzetteAccettatePolizia();
        gameData.pagaMulta(gameData.getSaldo() * 0.6);
        personaControllata = new PersonaControllata(personaDaControllare, true, TipoPersonaControllata.POLIZIOTTO);
        personeControllate.add(personaControllata);
    }

    /**
     * Metodo che gestisce la mazzetta di un rivoluzionario.
     *
     * @param personaDaControllare la persona da controllare
     * @param gameData i dati di gioco
     * @param personeControllate la lista delle persone controllate
     */
    private static void mazzettaRivoluzionario(Persona personaDaControllare, GameData gameData, ArrayList<PersonaControllata> personeControllate) {
        PersonaControllata personaControllata;
        System.out.println("Hai accettato la mazzetta, ti è andata bene ... era un rivoluzionario");
        gameData.incrementaRivoluzionariAccettati();
        personaControllata = new PersonaControllata(personaDaControllare, true, TipoPersonaControllata.RIVOLUZIONARIO);
        personeControllate.add(personaControllata);
    }

    /**
     * Metodo che gestisce i dati corretti rifiutati.
     *
     * @param personaDaControllare la persona da controllare
     * @param personeControllate la lista delle persone controllate
     * @param gameData i dati di gioco
     */
    private static void datiCorrettiRifiutati(Persona personaDaControllare, ArrayList<PersonaControllata> personeControllate, GameData gameData) {
        System.out.println("La persona aveva i dati corretti ... come sei salato");
        PersonaControllata personaControllata = new PersonaControllata(personaDaControllare, false, TipoPersonaControllata.PERSONA_NORMALE);
        personeControllate.add(personaControllata);
        if (Math.random() < 0.5){
            System.out.println("Sei stato fortunato, non hai presso la multa");
        } else {
            System.out.println("Hai preso una multa da 150§");
            gameData.pagaMulta(150);
        }
    }

    /**
     * Metodo che gestisce i dati errati accettati.
     *
     * @param personaDaControllare la persona da controllare
     * @param personeControllate la lista delle persone controllate
     * @param gameData i dati di gioco
     */
    private static void datiSbagliatiAccettati(Persona personaDaControllare, ArrayList<PersonaControllata> personeControllate, GameData gameData) {
        System.out.println("La persona aveva i dati errati");
        PersonaControllata personaControllata = new PersonaControllata(personaDaControllare, true, TipoPersonaControllata.PERSONA_NORMALE);
        personeControllate.add(personaControllata);
        if (Math.random() < 0.5){
            System.out.println("Sei stato fortunato, non hai presso la multa");
        } else {
            System.out.println("Hai preso una multa da 300§");
            gameData.pagaMulta(300);
        }
    }

    /**
     * Metodo che gestisce i dati corretti accettati.
     *
     * @param personaDaControllare la persona da controllare
     * @param personeControllate la lista delle persone controllate
     */
    private static void datiCorrettiAccettati(Persona personaDaControllare, ArrayList<PersonaControllata> personeControllate) {
        System.out.println("La persona aveva i dati corretti");
        PersonaControllata personaControllata = new PersonaControllata(personaDaControllare, true, TipoPersonaControllata.PERSONA_NORMALE);
        personeControllate.add(personaControllata);
    }

    /**
     * Metodo che termina il gioco e mostra il risultato finale.
     *
     * @param gameData i dati di gioco
     */
    private static void endGame(GameData gameData) {
        gameData.saveDiario();

        switch (gameData.getStatoGioco()){
            case ARRESTATO -> System.out.println("Sei stato arrestato");
            case GIUSTIZIATO -> System.out.println("Sei stato giustiziato");
            case RIVOLUZIONE -> System.out.println("Hai vinto la rivoluzione");
            case DODICESIMO_GIORNO -> System.out.println("Hai vinto, sei sopravvissuto 12 giorni");
            case NON_ABBASTANZA_SOLDI -> System.out.println("Non hai abbastanza soldi");
            default -> System.out.println("Hai perso");
        }
    }

    /**
     * Metodo che controlla se una persona ha i dati corretti.
     *
     * @param persona la persona da controllare
     * @param comuni la lista dei comuni
     * @param gameData i dati di gioco
     * @return true se la persona ha i dati corretti, false altrimenti
     */
    private static boolean controllaPersona(Persona persona, Comuni comuni, GameData gameData) {
        return persona.isCodiceFiscaleValido(comuni) && persona.isDocumentoValido(gameData.getData());
    }

    /**
     * Metodo che chiede all'utente di scegliere il formato di salvataggio.
     *
     * @return il formato scelto
     */
    private static Formato getFormato() {
        Formato formato = null;

        int sceltaFormato = InputData.readIntegerBetween("Inserisci il formato (0: JSON, 1: XML): ", 0, 1);
        if (sceltaFormato == 0) {
            System.out.println("Hai scelto il formato JSON");
            formato = Formato.JSON;
        } else {
            System.out.println("Hai scelto il formato XML");
            formato = Formato.XML;
        }
        return formato;
    }
}
